local harbour = ...

--------------------------------------------------------------------------
-- Apply building asset processor
harbour:registerAssetProcessor("models/lightTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register LargeGate prefab nodes
harbour:registerAssetId("models/lightTower.fbx/Prefab/lightTower/step_0_Light_RootPart", HarbourPrefix .. "_LIGHT_TOWER_ROOT_PREFAB")
harbour:registerAssetId("models/lightTower.fbx/Prefab/lightTower/step_1_Light_Tiling1Part", HarbourPrefix .. "_LIGHT_TOWER_TILING1_PREFAB")
harbour:registerAssetId("models/lightTower.fbx/Prefab/lightTower/step_2_Light_Tiling2Part", HarbourPrefix .. "_LIGHT_TOWER_TILING2_PREFAB")
harbour:registerAssetId("models/lightTower.fbx/Prefab/lightTower/step_3_Light_TopPart", HarbourPrefix .. "_LIGHT_TOWER_TOP_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	harbour:register({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = _NoSandbox*1 }
		    },
		    RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = _NoSandbox*2 }
		    }
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	        }
	})
end

local defaultNodePrefixList = {
	HarbourPrefix .. "_LIGHT_TOWER_TOP",
	HarbourPrefix .. "_LIGHT_TOWER_TILING1",
	HarbourPrefix .. "_LIGHT_TOWER_TILING2"
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

-- Register the light tower root part
harbour:register({
	DataType = "BUILDING_PART",
	Id = HarbourPrefix .. "_LIGHT_TOWER_ROOT_PART",
	Name = HarbourPrefix .. "_LIGHT_TOWER_ROOT_NAME",
	Description = HarbourPrefix .. "_LIGHT_TOWER_ROOT_DESC",
	Category = "CORE",
        BuildingZone = _LightTowerBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = HarbourPrefix .. "_LIGHT_TOWER_ROOT_PREFAB",
		EndPart = HarbourPrefix .. "_LIGHT_TOWER_TOP_PART",
		FillerList = {
			HarbourPrefix .. "_LIGHT_TOWER_TILING2_PART"
                },
		BasementFillerList = {
			HarbourPrefix .. "_LIGHT_TOWER_TILING1_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = _NoSandbox*0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = _NoSandbox*2 }
		}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 9
})

--------------------------------------------------------------------------
-- Register grass clearing components

harbour:registerPrefabComponent("models/lightTower.fbx/Prefab/lightTower/step_0_Light_RootPart", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 4,
        SoftRadius = 4,
        IsPermanent = true
})

-- Register Roof materials
harbour:registerAssetId("models/lightTower.fbx/Materials/redRoof", HarbourPrefix .. "_RED_ROOF_MATERIAL")
harbour:registerAssetId("models/lightTower.fbx/Materials/blueRoof", HarbourPrefix .. "_BLUE_ROOF_MATERIAL")
harbour:registerAssetId("models/lightTower.fbx/Materials/brownRoof", HarbourPrefix .. "_BROWN_ROOF_MATERIAL")
harbour:registerAssetId("models/lightTower.fbx/Materials/greyRoof", HarbourPrefix .. "_GREY_ROOF_MATERIAL")
harbour:registerAssetId("models/lightTower.fbx/Materials/grey2Roof", HarbourPrefix .. "_GREY2_ROOF_MATERIAL")

function registerRoofPart(_nameFbx, _nameAsset)
    harbour:registerAssetId("models/lightTower.fbx/Prefab/lightTower/" .. _nameFbx .. "Part", HarbourPrefix .. "_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration roof part
    harbour:register({
	DataType = "BUILDING_PART",
	Id = HarbourPrefix .. "_" .. _nameAsset .. "_PART",
	Name = HarbourPrefix .. "_" .. _nameAsset .. "_NAME",
	Description = HarbourPrefix .. "_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = HarbourPrefix .. "_" .. _nameAsset .. "_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = _NoSandbox*0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = _NoSandbox*5 }
		}
	}
    })

    harbour:registerPrefabComponent("models/lightTower.fbx/Prefab/lightTower/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "MINOR" -- Snaps the decoration only on minor attach nodes (here only on the roofs)
    })
end

registerRoofPart( "LightRoofA", "LIGHT_ROOF_A" )

-- Register the boat part
-- Apply building asset processor
harbour:registerAssetProcessor("models/boat.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerBoat(_namePart, _assetId, _behaviorWork, _capacity)
    -- Register IDs for all prefab nodes referenced here
    harbour:registerAssetId("models/boat.fbx/Prefab/harbour/" .. _namePart, HarbourPrefix .. "_" .. _assetId .. "_PREFAB")

    harbour:register({
	DataType = "BUILDING_PART",
	Id = HarbourPrefix .. "_" .. _assetId .. "_PART",
	Name = HarbourPrefix .. "_" .. _assetId .. "_NAME",
	Description = HarbourPrefix .. "_" .. _assetId .. "_DESC",
	Category = "DECORATION",
        BuildingZone = { ZoneEntryList = {
            { Polygon = polygon.createRectangle( {12, 4} ),
              Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } },
            { Polygon = polygon.createRectangle( {12, 4} ),
              Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } }
        }},
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = HarbourPrefix .. "_" .. _assetId .. "_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = _NoSandbox*1 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = _NoSandbox*5 }
		}
	},
	AssetBuildingFunction = HarbourPrefix .. "_FUNCTION_CORE_" .. _assetId
    })

    harbour:register({
		DataType = "BUILDING_FUNCTION_WORKPLACE",
                Id = HarbourPrefix .. "_FUNCTION_CORE_" .. _assetId,
		WorkerCapacity = _capacity,
                RelatedJob = { Job = HarbourPrefix .. "_SAILOR", Behavior = _behaviorWork }    })
end

registerBoat("boatStaticPart", "BOAT_STATIC", "BEHAVIOR_WORK", 1)

function registerCompBoat(_namePart, _assetId, _capacity, _sailingAnimation, _rowingAnimation)
    registerBoat(_namePart, _assetId, HarbourPrefix .. "_BEHAVIOR_TREE", _capacity )
    harbour:registerAssetId("models/boat.fbx/Prefab/harbour/" .. _namePart .. "/bateau", HarbourPrefix .. "_" .. _assetId .. "_BATEAU_PREFAB")

    -- Register boat component
    harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/" .. _namePart .. "/bateau", {
	DataType = HarbourPrefix .. "_COMP_BOAT",
        RandomPath = true,
        -- TODO: manage the collision between 2 boats
 	FlagTerrain = 2 ^ OBJECT_FLAG.TERRAIN:toNumber(), -- + 2 ^ OBJECT_FLAG.BUILDING_PART:toNumber() + 2 ^ OBJECT_FLAG.PLATFORM:toNumber()
        MinimalSailors = _capacity,
        SailingAnimation = _sailingAnimation,
        RowingAnimation = _rowingAnimation,
	Enabled = false
    })
    for i=1,_capacity do
        harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/" .. _namePart .. "/bateau/guardLocator" .. i, {
	    DataType = HarbourPrefix .. "_COMP_GUARDING_LOCATION" .. i,
	    Enabled = false
        })
    end
    harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/" .. _namePart, {
	DataType = "COMP_GROUNDED",
	GroundToWater = true
    })

end

registerCompBoat("boatPart", "BOAT", 1, true, false)
registerCompBoat("rowingPart", "ROWING", 2, false, true)
-- Register sail component
harbour:registerAssetId("models/boat.fbx/Prefab/harbour/boatPart/bateau/voile", HarbourPrefix .. "_BOAT_VOILE_PREFAB")
harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/boatPart/bateau/voile", {
	DataType = HarbourPrefix .. "_COMP_SAIL",
        SailScaleFactor = 4.0,
	Enabled = false
})
harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/boatPart", {
	    DataType = "COMP_BUILDING_PART",
	    FeedbackComponentListToActivate = {
		{ HarbourPrefix .. "_BOAT_BATEAU_PREFAB", HarbourPrefix .. "_COMP_BOAT" },
		{ HarbourPrefix .. "_BOAT_VOILE_PREFAB", HarbourPrefix .. "_COMP_SAIL" }
	    }
})

-- Register Oar component
harbour:registerAssetId("models/boat.fbx/Prefab/harbour/rowingPart/bateau/oars1", HarbourPrefix .. "_ROWING_OARS1_PREFAB")
harbour:registerAssetId("models/boat.fbx/Prefab/harbour/rowingPart/bateau/oars2", HarbourPrefix .. "_ROWING_OARS2_PREFAB")
harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/rowingPart/bateau/oars1", {
	DataType = HarbourPrefix .. "_COMP_OAR",
	Enabled = false
})
harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/rowingPart/bateau/oars2", {
	DataType = HarbourPrefix .. "_COMP_OAR",
	Enabled = false
})
harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/rowingPart", {
	    DataType = "COMP_BUILDING_PART",
	    FeedbackComponentListToActivate = {
		{ HarbourPrefix .. "_ROWING_BATEAU_PREFAB", HarbourPrefix .. "_COMP_BOAT" },
		{ HarbourPrefix .. "_ROWING_OARS1_PREFAB", HarbourPrefix .. "_COMP_OAR" },
		{ HarbourPrefix .. "_ROWING_OARS2_PREFAB", HarbourPrefix .. "_COMP_OAR" }
	    }
})

-- Register static boat component
harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/boatStaticPart", {
	DataType = HarbourPrefix .. "_COMP_STATIC_BOAT",
	Enabled = true
})
harbour:registerPrefabComponent("models/boat.fbx/Prefab/harbour/boatStaticPart", {
	DataType = "COMP_GROUNDED",
	GroundToWater = true
})

-- Register the dock part
-- Apply building asset processor
harbour:registerAssetProcessor("models/dock.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

function registerDockPart(_nameFbx, _nameAsset, _plankQuantity, _buildingZone, _category)

    harbour:registerAssetId("models/dock.fbx/Prefab/harbour/" .. _nameFbx .. "Part", HarbourPrefix .. "_" .. _nameAsset .. "_PREFAB")

    harbour:register({
	DataType = "BUILDING_PART",
	Id = HarbourPrefix .. "_" .. _nameAsset .. "_PART",
	Name = HarbourPrefix .. "_" .. _nameAsset .. "_NAME",
	Description = HarbourPrefix .. "_" .. _nameAsset .. "_DESC",
	Category = _category,
        BuildingZone = _buildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = HarbourPrefix .. "_" .. _nameAsset .. "_PREFAB"
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = _NoSandbox*0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = _NoSandbox*_plankQuantity }
		}
	}
    })

    harbour:registerPrefabComponent("models/dock.fbx/Prefab/harbour/" .. _nameFbx .. "Part", {
	DataType = "COMP_GROUNDED",
	GroundToWater = true
    })
end

registerDockPart("dock", "DOCK", 5, _DockBuildingZone, "DECORATION")
registerDockPart("ponton", "PONTOON", 3, _PontonBuildingZone, "DECORATION")
registerDockPart("ramp", "RAMP", 2, _RampBuildingZone, "DOOR")

harbour:registerPrefabComponent("models/dock.fbx/Prefab/harbour/rampPart", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = "DOOR"
})

--------------------------------------------------------------------------
-- Configure all PLATFORM colliders
harbour:configurePrefabFlagList("models/boat.fbx/Prefab/harbour/boatPart/bateau", { "PLATFORM" })
harbour:configurePrefabFlagList("models/boat.fbx/Prefab/harbour/rowingPart/bateau", { "PLATFORM" })
harbour:configurePrefabFlagList("models/boat.fbx/Prefab/harbour/boatStaticPart/bateau", { "PLATFORM" })
harbour:configurePrefabFlagList("models/dock.fbx/Prefab/harbour/dockPart", { "PLATFORM" })
harbour:configurePrefabFlagList("models/dock.fbx/Prefab/harbour/pontonPart", { "PLATFORM" })
harbour:configurePrefabFlagList("models/dock.fbx/Prefab/harbour/rampPart", { "PLATFORM" })
